# -*- coding: utf-8 -*-
import codecs
import datetime
import email
import logging
import os
import re

from django.conf import settings
from django.utils import timezone
from django.utils.crypto import get_random_string
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _

# TODO: thx package logger
logger = logging.getLogger('file')


regexp = r'(?P<email>[\w\.-]+@[\w\.-]+\.\w+)'


class EmailParser:
    """Pay attention, Parser works with mbx format email"""

    def __init__(self, filename=None, email_payload=None):
        # TODO: check filename or email, not both
        self.filename = filename
        self.email_payload = email_payload
        self.msg = self.get_msg()
        self.return_path = self.get_return_path()
        self.message_id = self.get_message_id()
        self.references = self.get_references()
        self.in_reply_to = self.get_in_reply_to()
        self.date = self.get_date()
        self.sender = self.get_sender()
        self.recipients = self.get_recipients()
        self.to_recipients = self.get_to_recipients()
        self.cc_recipients = self.get_cc_recipients()
        self.bcc_recipients = self.get_bcc_recipients()
        self.receiveds = self.get_receiveds()
        self.subject = self.get_subject()
        self.body = self.get_body()
        self.attachments = self.get_attachments()

    def get_msg(self):
        """
        Return an email.Message reading filename or stdin
        """
        if not self.filename:
            assert self.email_payload, "email_payload is required"
            try:
                now = timezone.now().astimezone().strftime('%Y.%m.%d-%H:%M:%S%f-%Z')
            except Exception:
                now = datetime.datetime.now().astimezone().strftime('%Y.%m.%d-%H:%M:%S%f-%Z')
            # self.filename = f'/tmp/email{now}.mbox'
            self.filename = settings.THX_EMAIL_PARSER_FILE_PATH_NAME_STRUCTURE.format(timezone_now=now)
            with open(self.filename, 'wb') as f:
                # Don't interpret the encoding!
                if self.email_payload:
                    f.write(self.email_payload.read())
        try:
            with open(self.filename, 'r') as file:
                msg = email.message_from_file(file)
                file.seek(0)
                self.email_payload = file.read(8192)
        except Exception as e:
            logger.debug(_("import con codec Latin-1"))
            try:
                msg = email.message_from_file(codecs.open(self.filename, 'r', 'latin-1'))
            except Exception as e2:
                logger.error(f"ParseEmail get_msg() error: {e2} - ORIGINAL ERROR {e}")
                raise e2
        return msg

    def get_return_path(self):
        """
        get Return-paht address header
        :return:str address
        """
        return self.msg.get("return-path", "").strip("<>").lower()  # TODO if "" gestire come None?

    def get_from_address(self):
        """
        Return the address of from header
        :return: str address
        """
        _from = self.msg.get('from', '').lower()
        if "<" in _from:
            return _from.split("<")[1].strip(">").lower()
        return _from

    def get_reply_to(self):
        """
        Return the address of reply-to header
        :return: str address
        """
        _reply_to = self.msg.get('reply-to', '').lower()
        if "<" in _reply_to:
            return _reply_to.split("<")[1].strip("<>").lower()
        return

    @staticmethod
    def get_domain(address: str):
        param_list = address.split("@")
        return param_list[1] if len(param_list) > 1 else ""

    @staticmethod
    def get_treacherous_words_list() -> list:
        return []

    def get_dict(self):
        return dict(self.msg)

    def get_message_id(self):
        return self.msg.get('message-id', None)

    def get_references(self):
        if self.msg.get('references', None):
            return self.msg.get('references', '').replace('None', '').replace('\n', '')
        return ''

    def get_in_reply_to(self):
        return self.msg.get('in-reply-to', None)

    def get_date(self):
        date = self.msg.get('date', None)
        return email.utils.parsedate(date)

    def get_datetime_tz(self):
        date = self.msg.get('date', None)
        if date:
            return email.utils.parsedate_to_datetime(date)
        # TODO VERIFICARE
        return ""

    def get_subject(self):
        """
        Return subject decoded according to it's charset
        """
        subject = self.msg.get('Subject', None)

        if subject:
            try:
                decoded_header = email.header.decode_header(subject)
                decoded_subject = decoded_header[0][0]
                if isinstance(decoded_subject, str):
                    return decoded_subject
                try:
                    return decoded_subject.decode('utf-8')
                except Exception:
                    return decoded_subject.decode('latin1')

            except Exception as e:
                raise e
        return subject

    def get_part_count(self):
        """
        Return part count if it is multipart message
        :return: int or None
        """
        if self.msg.is_multipart():
            return sum(1 for _ in self.msg.walk())
        return None

    def get_part_type(self):
        """
        Return a content type list if it is multipart message
        :return: list or None
        """
        if self.msg.is_multipart():
            return [part.get_content_type() for part in self.msg.walk()]
        return None

    def get_part_dict(self):
        """
        Return a list of dictionary if it is multipart message
        :return: list({'type': type, 'decoded': payload}) or None
        """
        if self.msg.is_multipart():
            part_list = []
            for part in self.msg.walk():
                payload = None
                try:
                    payload = self.get_decoded_payload(part)
                except:  # TODO: gestire eccezioni
                    pass
                part_list.append({
                    'type': part.get_content_type(),
                    'decoded': payload
                })
            return part_list
        return None

    def get_body(self):
        """
        Get body of email. If present prefer text/html over text/plain
        """
        body = ""
        if self.msg.is_multipart():
            # let up give precedence to html
            for part in self.msg.walk():
                ctype = part.get_content_type()
                cdispo = str(part.get('Content-Disposition', ""))

                if ctype == 'text/html' and 'attachment' not in cdispo:
                    body = f"{body} \n {self.get_decoded_payload(part)}"
                    continue

                if ctype == 'text/plain':
                    body = f"{body} \n {self.get_decoded_payload(part)}"
                    continue

                if ctype == 'multipart/related':
                    # explicit escape of this multipart/related https://docs.python.org/3/library/email.message.html
                    continue

            m = re.search('<body[^>]*>(?P<body>.+)</body', body, flags=re.DOTALL)
            if m:
                body_html = m.group('body')
            else:
                body_html = '<div class="pre">{body}</div>'.format(body=body)

        # not multipart - i.e. plain text, no attachments, keeping fingers crossed
        else:
            body = self.get_decoded_payload(self.msg)
            body_html = '<div class="pre">{body}</div>'.format(body=body)

        return mark_safe(body_html) if body_html else ''

    def get_sender(self):
        """
        Return the address of the sender
        :return: string
        """
        email_from = self.msg.get('From', None)
        return re.search(regexp, email_from.lower()).group('email')

    def get_recipients(self):
        """
        Return the address of the all recipients
        :return: email list
        """
        return self.get_to_recipients() + self.get_cc_recipients() + self.get_bcc_recipients()

    def get_to_recipients(self):
        """
        Return the address of the recipients
        :return: email list
        """
        email_tos = self.msg.get('to', None)
        if email_tos:
            return [
                re.search(regexp, email_to.lower()).group('email') for email_to in email_tos.split(',')
                if re.search(regexp, email_to.lower())
            ]
        return []

    def get_cc_recipients(self):
        """
        Return the address of the recipients
        :return: email list
        """
        email_tos = self.msg.get('cc', None)
        if email_tos:
            return [
                re.search(regexp, email_to.lower()).group('email') for email_to in email_tos.split(',')
                if re.search(regexp, email_to.lower())
            ]
        return []

    def get_bcc_recipients(self):
        """
        Return the address of the recipients
        :return: email list
        """
        email_tos = self.msg.get('bcc', None)
        if email_tos:
            return [
                re.search(regexp, email_to.lower()).group('email') for email_to in email_tos.split(',')
                if re.search(regexp, email_to.lower())
            ]
        return []

    def get_receiveds(self):
        """
        Return ordered Receiveds in Header message
        :return: string list, example: [
            "from wash02.thundersystems.it (wash02.thundersystems.it\n [212.31.253.60])\n\tby localhost.localdomain
            (Postfix) with ESMTPS id 05D441146\n\tfor <code@tickets.thux.dev>; Tue,  9 Feb 2021 10:33:44 +0100 (CET)"
        """
        return self.msg.get_all('Received', [])

    @staticmethod
    def _get_receiveds_list_dicts(receiveds: list):
        """
        this is a private method because is strongly tied to structure of received from Thux email server,
        if structure change, the method could be break.
        get receiveds as a list of dict params with from, by, and to params
        example: [{'from': ' wash02.thundersystems.it (wash02.thundersystems.it\n [212.31.253.60])\n\t',
        'by': ' localhost.localdomain (Postfix) with ESMTPS id 3109E1146\n\t',
        'for': ' <code@tickets.thux.dev>; Tue,  9 Feb 2021 13:02:26 +0100 (CET)'},
        {'from': ' [212.31.253.10] (helo=zimbra.thux.it)\n\t',
        'by': ' wash02.thundersystems.it with esmtps\n
        (TLS1.2:ECDHE_RSA_AES_256_GCM_SHA384:256)\n\t(Baruwa 2.0)\n\t(envelope-from <parenti.vittorino@gmail.com>)\n\
        tid 1l9Riq-0001Cp-81 ret-id none;\n\t',
        'for': ' code@tickets.thux.dev; Tue, 09 Feb 2021 13:02:24 +0100'},
        {'from': ' localhost (localhost [127.0.0.1])\n\t',
        'by': ' zimbra.thux.it (Postfix) with ESMTP id BDE15F84496\n\t',
        'for': ' <code@tickets.thux.dev>; Tue,  9 Feb 2021 13:02:23 +0100 (CET)'},
        {'from': ' zimbra.thux.it ([127.0.0.1])\n\t',
        'by': ' localhost (zimbra.thux.it [127.0.0.1]) (amavisd-new, port 10024)\n\twith ESMTP id V4t3XYYMUSCv ',
        'for': ' <code@tickets.thux.dev>;\n\tTue,  9 Feb 2021 13:02:23 +0100 (CET)'},
        {'from': ' wash01.thundersystems.it (wash01.thundersystems.it\n [213.215.191.74])\n\t',
        'by': ' zimbra.thux.it (Postfix) with ESMTPS id A8AC4F84482\n\t',
        'for': ' <issues@thux.it>; Tue,  9 Feb 2021 13:02:23 +0100 (CET)'},
        {'from': ' mail-ej1-f47.google.com ([209.85.218.47])\n\t',
        'by': ' wash01.thundersystems.it with esmtps\n (TLS1.2:ECDHE_RSA_AES_128_GCM_SHA256:128)\n\t
        (Baruwa 2.0)\n\t(envelope-from <parenti.vittorino@gmail.com>)\n\tid 1l9Rik-0005td-BV ret-id none;\n\t',
        'for': ' issues@thux.it; Tue, 09 Feb 2021 13:02:18 +0100'}]
        :param receiveds: list of str(receveds)
        :return: list of dicts

        """
        receiveds_list_dicts = []
        for received in receiveds:
            params_dict = {}
            try:
                step1 = received.split("by")
                params_dict["from"] = step1[0].split('from')[1]
                step2 = step1[1].split('for')
                params_dict["by"] = step2[0]
                params_dict["for"] = step2[1]
                receiveds_list_dicts.append(params_dict)
            except IndexError:
                """per il momento se la struttura è diversa va in errore e salto quel received"""
                pass
        return receiveds_list_dicts

    def get_allowed_myme_type_extensions(self, content_type: str):
        return self.CONTENT_TYPE_MYME_TYPE_ALLOWED.get(content_type, {}).get("extensions")

    @staticmethod
    def get_payload_as_bytes(part):
        payload = part.get_payload(decode=True) if part.get_payload(decode=True) else part.get_payload()
        try:
            if isinstance(payload, bytes):
                pass
            elif isinstance(payload, str):
                payload = payload.encode()
            elif isinstance(payload, list):
                if isinstance(payload[0], email.message.Message):
                    try:
                        payload = payload[0].as_bytes()
                    except UnicodeEncodeError:
                        payload = payload[0].as_string()
                        payload = payload.encode()
        except Exception:
            # no action needed because error will be intercepted later with more ticket and email references
            pass
        return payload

    def append_attachment(self, attachments_container, part, is_inline=False):
        attachment = part.get_filename()
        if not attachment:
            attachment = get_random_string(length=12)
        # extensions = self.CONTENT_TYPE_MYME_TYPE_ALLOWED.get(part.get('Content-Type')).get("extensions")
        # content_type = part.get('Content-Type').split(";")[0]
        content_type = part.get_content_type()
        extensions = self.get_allowed_myme_type_extensions(content_type)
        if content_type not in [
            "text/html", "image/jpeg", "image/tiff", "audio/midi audio/x-midi"
            # "text/javascript",
        ]:
            extension = extensions[0]
        else:
            # TODO si potrebbe riportare come testo inserendo una riga iniziale come "original extension: <estensione>"
            #  così da poterla riaprire in un secondo momento
            extension = extensions[0]

        if bool(attachment):
            attachment = {
                'name': attachment,
                'extension': extension,
                'bytes': self.get_payload_as_bytes(part),
                # 'default_type_as_str': part.get_payload(),
                'content_type': content_type,
                'ID': part.get('Content-ID'),
                'content_disposition': part.get('Content-Disposition'),
                'is_inline': is_inline
            }
            attachments_container.append(attachment)

    def get_attachments(self):
        """
        Return a list of dict of all the attachment with
        name of the file and byte content
        """
        attachments = []
        for part in self.msg.walk():
            if part.get_content_maintype() == 'multipart':
                continue
            # if (
            #         self.CONTENT_TYPE_MYME_TYPE_ALLOWED.get(part.get_content_type(), None)
            #         # part.get_content_type() in self.CONTENT_TYPE_MYME_TYPE_ALLOWED and
            #
            #         # or part.get('Content-Disposition').startswith('attachment')
            # ):
            if (
                    self.CONTENT_TYPE_MYME_TYPE_ALLOWED.get(part.get_content_type(), None)
                    and part.get_content_disposition() == 'attachment'
            ):
                self.append_attachment(attachments, part)

        return attachments

    def get_body_attachments(self):
        """
        this method includes body type attachments like image into self.attachments list.

        why get_body_attachments if is really similar to get_attachments?!
        because this type of attachments is needed only if you want to get img e other things from email body
        and this is not a standard behavior.
        :return:
        """
        attachments = []
        for part in self.msg.walk():
            if part.get_content_maintype() == 'multipart':
                continue
            if (
                    self.CONTENT_TYPE_MYME_TYPE_ALLOWED.get(part.get_content_type(), None)
                    and part.get_content_disposition() != 'attachment'
                    # and part.get_content_type() != 'text/plain'?
            ):
                self.append_attachment(attachments, part, is_inline=True)

        self.attachments.extend(attachments)

    def get_decoded_payload(self, part):
        try:
            return part.get_payload(decode=True).decode(part.get_content_charset())
        except UnicodeDecodeError:
            # Incomprensibile... ma sto avendo problemi
            return part.get_payload(decode=True).decode('latin-1')
        except TypeError:
            return part.get_payload(decode=True).decode("utf-8")
        except Exception as e:
            return f"PARSER ERROR: {e}"

    def write_attachments(self, path="/tmp/"):
        """
        Return a list of all the attachment streams written on
        /tmp/ folder
        """
        attachments = []
        for part in self.msg.walk():
            if part.get_content_maintype() == 'multipart':
                continue
            if part.get('Content-Disposition') is None:
                continue
            if not part.get('Content-Disposition').startswith('attachment'):
                continue
            attachment = part.get_filename()
            if bool(attachment):
                file_path = os.path.join(path, attachment)
                if not os.path.isfile(file_path):
                    fp = open(file_path, 'wb')
                    fp.write(part.get_payload(decode=True))
                    fp.close()
                    attachments.append(fp)
                else:
                    fp = open(file_path, 'rb')
                    attachments.append(fp)
        return attachments

    CONTENT_TYPE_MYME_TYPE_ALLOWED = {
        "audio/aac": {"extensions": (".aac",), "description": "AAC audio", "other_info": ""},
        "application/x-abiword": {"extensions": (".abw",), "description": "AbiWord document", "other_info": ""},
        "application/x-freearc": {"extensions": (".arc",), "description": "Archive document (multiple files embedded)",
                                  "other_info": ""},
        "video/quicktime": {"extensions": (".mov",), "description": "", "other_info": ""},
        "video/x-msvideo": {"extensions": (".avi",), "description": "AVI: Audio Video Interleave", "other_info": ""},
        "application/vnd.amazon.ebook": {"extensions": (".azw",), "description": "Amazon Kindle eBook format",
                                         "other_info": ""},
        "application/octet-stream": {"extensions": (".bin",), "description": "Any kind of binary data",
                                     "other_info": ""},
        "image/bmp": {"extensions": (".bmp",), "description": "Windows OS/2 Bitmap Graphics", "other_info": ""},
        "application/x-bzip": {"extensions": (".bz",), "description": "BZip archive", "other_info": ""},
        "application/x-bzip2": {"extensions": (".bz2",), "description": "BZip2 archive", "other_info": ""},
        "application/x-csh": {"extensions": (".csh",), "description": "C-Shell script", "other_info": ""},
        "text/css": {"extensions": (".css",), "description": "Cascading Style Sheets (CSS)", "other_info": ""},
        "text/csv": {"extensions": (".csv",), "description": "Comma-separated values (CSV)", "other_info": ""},
        "application/msword": {"extensions": (".doc",), "description": "Microsoft Word", "other_info": ""},
        "application/vnd.openxmlformats-officedocument.wordprocessingml.document": {
            "extensions": (".docx",),
            "description": "Microsoft Word (OpenXML)",
            "other_info": ""
        },
        "application/vnd.ms-fontobject": {"extensions": (".eot",), "description": "MS Embedded OpenType fonts",
                                          "other_info": ""},
        "application/epub+zip": {"extensions": (".epub",), "description": "Electronic publication (EPUB)",
                                 "other_info": ""},
        "application/gzip": {"extensions": (".gz",), "description": "GZip Compressed Archive", "other_info": ""},
        "image/gif": {"extensions": (".gif",), "description": "Graphics Interchange Format (GIF)", "other_info": ""},
        "text/html": {"extensions": (".html", ".htm ",), "description": "HyperText Markup Language (HTML)",
                      "other_info": ""},
        "image/vnd.microsoft.icon": {"extensions": (".ico",), "description": "Icon format", "other_info": ""},
        "text/calendar": {"extensions": (".ics", ".mjs",), "description": "iCalendar format", "other_info": ""},
        "application/java-archive": {"extensions": (".jar",), "description": "Java Archive (JAR)", "other_info": ""},
        "image/jpeg": {"extensions": (".jpeg .jpg",), "description": "JPEG images", "other_info": ""},
        "text/javascript": {"extensions": (".js",), "description": "JavaScript or JavaScript module", "other_info": ""},
        # per the following specifications: https://html.spec.whatwg.org/multipage/#scriptingLanguages
        # https://html.spec.whatwg.org/multipage/#dependencies:willful-violation
        # https://datatracker.ietf.org/doc/draft-ietf-dispatch-javascript-mjs/
        "application/json": {"extensions": (".json",), "description": "JSON format", "other_info": ""},
        "application/ld+json": {"extensions": (".jsonld",), "description": "JSON-LD format", "other_info": ""},
        "audio/midi audio/x-midi": {"extensions": (".mid .midi",),
                                    "description": "Musical Instrument Digital Interface (MIDI)", "other_info": ""},
        "audio/mpeg": {"extensions": (".mp3",), "description": "MP3 audio", "other_info": ""},
        "application/x-cdf": {"extensions": (".cda",), "description": "MP3 CD audio", "other_info": ""},
        "video/mp4": {"extensions": (".mp4",), "description": "MP4 audio", "other_info": ""},
        "video/mpeg": {"extensions": (".mpeg",), "description": "MPEG Video", "other_info": ""},
        "application/vnd.apple.installer+xml": {"extensions": (".mpkg",), "description": "Apple Installer Package",
                                                "other_info": ""},
        "application/vnd.oasis.opendocument.presentation": {"extensions": (".odp",),
                                                            "description": "OpenDocument presentation document",
                                                            "other_info": ""},
        "application/vnd.oasis.opendocument.spreadsheet": {"extensions": (".ods",),
                                                           "description": "OpenDocument spreadsheet document",
                                                           "other_info": ""},
        "application/vnd.oasis.opendocument.text": {"extensions": (".odt",),
                                                    "description": "OpenDocument text document", "other_info": ""},
        "audio/ogg": {"extensions": (".oga",), "description": "OGG audio", "other_info": ""},
        "video/ogg": {"extensions": (".ogv",), "description": "OGG video", "other_info": ""},
        "application/ogg": {"extensions": (".ogx",), "description": "OGG", "other_info": ""},
        "audio/opus": {"extensions": (".opus",), "description": "Opus audio", "other_info": ""},
        "font/otf": {"extensions": (".otf",), "description": "OpenType font", "other_info": ""},
        "image/png": {"extensions": (".png",), "description": "Portable Network Graphics", "other_info": ""},
        "application/pdf": {"extensions": (".pdf",), "description": "Adobe Portable Document Format (PDF)",
                            "other_info": ""},
        "application/x-httpd-php": {"extensions": (".php",),
                                    "description": "Hypertext Preprocessor (Personal Home Page)", "other_info": ""},
        "application/vnd.ms-powerpoint": {"extensions": (".ppt",), "description": "Microsoft PowerPoint",
                                          "other_info": ""},
        "application/vnd.openxmlformats-officedocument.presentationml.presentation": {
            "extensions": (".pptx",),
            "description":
                "Microsoft PowerPoint (OpenXML)",
            "other_info": ""
        },
        "application/vnd.rar": {"extensions": (".rar",), "description": "RAR archive", "other_info": ""},
        "application/rtf": {"extensions": (".rtf",), "description": "Rich Text Format (RTF)", "other_info": ""},
        "application/x-sh": {"extensions": (".sh",), "description": "Bourne shell script", "other_info": ""},
        "image/svg+xml": {"extensions": (".svg",), "description": "Scalable Vector Graphics (SVG)", "other_info": ""},
        "application/x-shockwave-flash": {"extensions": (".swf",),
                                          "description": "Small web format (SWF) or Adobe Flash document",
                                          "other_info": ""},
        "application/x-tar": {"extensions": (".tar",), "description": "Tape Archive (TAR)", "other_info": ""},
        "image/tiff": {"extensions": (".tif .tiff",), "description": "Tagged Image File Format (TIFF)",
                       "other_info": ""},
        "video/mp2t": {"extensions": (".ts",), "description": "MPEG transport stream", "other_info": ""},
        "font/ttf": {"extensions": (".ttf",), "description": "TrueType Font", "other_info": ""},
        "text/plain": {"extensions": (".txt",), "description": "Text, (generally ASCII or ISO 8859-n)",
                       "other_info": ""},
        "application/vnd.visio": {"extensions": (".vsd",), "description": "Microsoft Visio", "other_info": ""},
        "audio/wav": {"extensions": (".wav",), "description": "Waveform Audio Format", "other_info": ""},
        "audio/webm": {"extensions": (".weba",), "description": "WEBM audio", "other_info": ""},
        "video/webm": {"extensions": (".webm",), "description": "WEBM video", "other_info": ""},
        "image/webp": {"extensions": (".webp",), "description": "WEBP image", "other_info": ""},
        "font/woff": {"extensions": (".woff",), "description": "Web Open Font Format (WOFF)", "other_info": ""},
        "font/woff2": {"extensions": (".woff2",), "description": "Web Open Font Format (WOFF)", "other_info": ""},
        "application/xhtml+xml": {"extensions": (".xhtml",), "description": "XHTML", "other_info": ""},
        "application/vnd.ms-excel": {"extensions": (".xls",), "description": "Microsoft Excel", "other_info": ""},
        "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet": {
            "extensions": (".xlsx",),
            "description": "Microsoft Excel (OpenXML)",
            "other_info": ""
        },
        "application/xml": {
            "extensions": (".xml",), "description": "XML",
            "other_info":
                "if not readable from casual users (RFC 3023, section 3) "
                "text/xml if readable from casual users (RFC 3023, section 3)"
        },
        "application/vnd.mozilla.xul+xml": {"extensions": (".xul",), "description": "XUL", "other_info": ""},
        "application/zip": {"extensions": (".zip",), "description": "ZIP archive", "other_info": ""},
        "video/3gpp audio/3gpp": {"extensions": (".3gp",), "description": "3GPP audio/video container",
                                  "other_info": "if it doesn't contain video"},
        "video/3gpp2 audio/3gpp2": {"extensions": (".3g2",), "description": "3GPP2 audio/video container",
                                    "other_info": "if it doesn't contain video"},
        "application/x-7z-compressed": {"extensions": (".7z",), "description": "7-zip archive", "other_info": ""},
        "message/rfc822": {
            "extensions": (".eml", ".mbox"),
            "description": "A Content-Type of 'message/rfc822' indicates that the body contains an encapsulated "
                           "message, with the syntax of an RFC 822 message."},
        "application/mbox": {"extensions": (".mbox",), "description": ""},
    }

    # source https://developer.mozilla.org/en-US/docs/Web/HTTP/Basics_of_HTTP/MIME_types/Common_types
    # altri: https://www.asciitable.it/mime-types
